@@ -3,9 +3,10 @@ import os
 import platform
 import shlex
 import socket
+import subprocess
 import time
 from dataclasses import dataclass
-from subprocess import Popen, PIPE, TimeoutExpired
+from subprocess import TimeoutExpired
 from typing import Any, Union
 
 import plogger
@@ -99,20 +100,70 @@ class Plinux:
 
         """
 
-        with Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE) as process:
-            # process.wait(timeout)
-            try:
-                self.logger.info(f'COMMAND: "{cmd}"')
-                stdout, stderr = process.communicate(timeout=timeout)
-                data = (stdout + stderr).decode().strip()
-                self.logger.info(f'RESULT: "{data}"')
-                return data
-            except TimeoutExpired:
-                process.kill()
-                stdout, stderr = process.communicate()
-                data = stdout + stderr
-                self.logger.exception('Connection timeout')
-                return data.decode()
+        try:
+            self.logger.info(f'COMMAND: "{cmd}"')
+            cmd_divided = shlex.split(cmd)
+            result = subprocess.run(cmd_divided, capture_output=True, timeout=timeout)
+
+            out = result.stdout.decode().strip()
+            err = result.stderr.decode().strip()
+            response = result.returncode, out, err, cmd
+
+            return ResponseParser(response)
+        except subprocess.TimeoutExpired:
+            self.logger.exception('Connection timeout')
+            raise TimeoutExpired
+
+    def run_cmd_local_native(self, *popenargs,
+                             stdin_input=None,
+                             capture_output: bool = False,
+                             timeout: int = 60,
+                             check: bool = False,
+                             **kwargs):
+        """It's just native subprocess' .run invocation.
+
+        The returned instance will have attributes args, returncode, stdout and
+        stderr. By default, stdout and stderr are not captured, and those attributes
+        will be None. Pass stdout=PIPE and/or stderr=PIPE in order to capture them.
+
+        If check is True and the exit code was non-zero, it raises a
+        CalledProcessError. The CalledProcessError object will have the return code
+        in the returncode attribute, and output & stderr attributes if those streams
+        were captured.
+
+        If timeout is given, and the process takes too long, a TimeoutExpired
+        exception will be raised.
+
+        There is an optional argument "stdin_input", allowing you to
+        pass bytes or a string to the subprocess's stdin.  If you use this argument
+        you may not also use the Popen constructor's "stdin" argument, as
+        it will be used internally.
+
+        By default, all communication is in bytes, and therefore any "stdin_input" should
+        be bytes, and the stdout and stderr will be bytes. If in text mode, any
+        "stdin_input" should be a string, and stdout and stderr will be strings decoded
+        according to locale encoding, or by "encoding" if set. Text mode is
+        triggered by setting any of text, encoding, errors or universal_newlines.
+
+        The other arguments are the same as for the Popen constructor.
+
+        :param popenargs:
+        :param stdin_input:
+        :param capture_output:
+        :param timeout:
+        :param check:
+        :param kwargs:
+        :return:
+        """
+
+        cmd = shlex.split(*popenargs)
+        cmd_to_log = ' '.join(cmd)
+        self.logger.info(f'COMMAND: "{cmd_to_log}"')
+
+        result = subprocess.run(cmd, input=stdin_input, capture_output=capture_output, timeout=timeout,
+                                check=check, **kwargs)
+
+        return result
 
     def _client(self, sftp: bool = False, timeout: int = 15):
         """https://www.paramiko.org/"""
